# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: muscle_manager_protocol.proto

import sys
_b=sys.version_info[0]<3 and (lambda x:x) or (lambda x:x.encode('latin1'))
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import timestamp_pb2 as google_dot_protobuf_dot_timestamp__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='muscle_manager_protocol.proto',
  package='muscle_manager_protocol',
  syntax='proto3',
  serialized_options=None,
  serialized_pb=_b('\n\x1dmuscle_manager_protocol.proto\x12\x17muscle_manager_protocol\x1a\x1fgoogle/protobuf/timestamp.proto\"\x90\x01\n\nLogMessage\x12\x13\n\x0binstance_id\x18\x01 \x01(\t\x12-\n\ttimestamp\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12\x30\n\x05level\x18\x04 \x01(\x0e\x32!.muscle_manager_protocol.LogLevel\x12\x0c\n\x04text\x18\x05 \x01(\t\"\x0b\n\tLogResult\"I\n\x04Port\x12\x0c\n\x04name\x18\x01 \x01(\t\x12\x33\n\x08operator\x18\x02 \x01(\x0e\x32!.muscle_manager_protocol.Operator\"\xa7\x02\n\x0cProfileEvent\x12\x13\n\x0binstance_id\x18\x01 \x01(\t\x12.\n\nstart_time\x18\x02 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12-\n\tstop_time\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.Timestamp\x12=\n\nevent_type\x18\x04 \x01(\x0e\x32).muscle_manager_protocol.ProfileEventType\x12+\n\x04port\x18\x05 \x01(\x0b\x32\x1d.muscle_manager_protocol.Port\x12\x13\n\x0bport_length\x18\x06 \x01(\x05\x12\x0c\n\x04slot\x18\x07 \x01(\x05\x12\x14\n\x0cmessage_size\x18\x08 \x01(\x03\"@\n\x07Profile\x12\x35\n\x06\x65vents\x18\x01 \x03(\x0b\x32%.muscle_manager_protocol.ProfileEvent\"\x0f\n\rProfileResult\"u\n\x13RegistrationRequest\x12\x15\n\rinstance_name\x18\x01 \x01(\t\x12\x19\n\x11network_locations\x18\x02 \x03(\t\x12,\n\x05ports\x18\x03 \x03(\x0b\x32\x1d.muscle_manager_protocol.Port\"b\n\x12RegistrationResult\x12\x35\n\x06status\x18\x01 \x01(\x0e\x32%.muscle_manager_protocol.ResultStatus\x12\x15\n\rerror_message\x18\x02 \x01(\t\"+\n\x07\x43onduit\x12\x0e\n\x06sender\x18\x01 \x01(\t\x12\x10\n\x08receiver\x18\x02 \x01(\t\"$\n\x0bPeerRequest\x12\x15\n\rinstance_name\x18\x01 \x01(\t\"\x9a\x03\n\nPeerResult\x12\x35\n\x06status\x18\x01 \x01(\x0e\x32%.muscle_manager_protocol.ResultStatus\x12\x15\n\rerror_message\x18\x02 \x01(\t\x12\x32\n\x08\x63onduits\x18\x03 \x03(\x0b\x32 .muscle_manager_protocol.Conduit\x12K\n\x0fpeer_dimensions\x18\x04 \x03(\x0b\x32\x32.muscle_manager_protocol.PeerResult.PeerDimensions\x12I\n\x0epeer_locations\x18\x05 \x03(\x0b\x32\x31.muscle_manager_protocol.PeerResult.PeerLocations\x1a\x37\n\x0ePeerDimensions\x12\x11\n\tpeer_name\x18\x01 \x01(\t\x12\x12\n\ndimensions\x18\x02 \x03(\x05\x1a\x39\n\rPeerLocations\x12\x15\n\rinstance_name\x18\x01 \x01(\t\x12\x11\n\tlocations\x18\x02 \x03(\t\"\x1e\n\x0cListOfDouble\x12\x0e\n\x06values\x18\x01 \x03(\x01\"K\n\x12ListOfListOfDouble\x12\x35\n\x06values\x18\x01 \x03(\x0b\x32%.muscle_manager_protocol.ListOfDouble\"\xb5\x02\n\x07Setting\x12\x0c\n\x04name\x18\x01 \x01(\t\x12=\n\nvalue_type\x18\x02 \x01(\x0e\x32).muscle_manager_protocol.SettingValueType\x12\x14\n\x0cvalue_string\x18\x03 \x01(\t\x12\x11\n\tvalue_int\x18\x04 \x01(\x03\x12\x13\n\x0bvalue_float\x18\x05 \x01(\x01\x12\x12\n\nvalue_bool\x18\x06 \x01(\x08\x12?\n\x10value_list_float\x18\x07 \x01(\x0b\x32%.muscle_manager_protocol.ListOfDouble\x12J\n\x15value_list_list_float\x18\x08 \x01(\x0b\x32+.muscle_manager_protocol.ListOfListOfDouble\"\x11\n\x0fSettingsRequest\"J\n\x0eSettingsResult\x12\x38\n\x0esetting_values\x18\x01 \x03(\x0b\x32 .muscle_manager_protocol.Setting\".\n\x15\x44\x65registrationRequest\x12\x15\n\rinstance_name\x18\x01 \x01(\t\"d\n\x14\x44\x65registrationResult\x12\x35\n\x06status\x18\x01 \x01(\x0e\x32%.muscle_manager_protocol.ResultStatus\x12\x15\n\rerror_message\x18\x02 \x01(\t*]\n\x0cResultStatus\x12\x19\n\x15RESULT_STATUS_SUCCESS\x10\x00\x12\x17\n\x13RESULT_STATUS_ERROR\x10\x01\x12\x19\n\x15RESULT_STATUS_PENDING\x10\x02*\x88\x01\n\x08Operator\x12\x11\n\rOPERATOR_NONE\x10\x00\x12\x13\n\x0fOPERATOR_F_INIT\x10\x01\x12\x10\n\x0cOPERATOR_O_I\x10\x02\x12\x0e\n\nOPERATOR_S\x10\x03\x12\x0e\n\nOPERATOR_B\x10\x04\x12\x10\n\x0cOPERATOR_O_F\x10\x05\x12\x10\n\x0cOPERATOR_MAP\x10\x06*w\n\x08LogLevel\x12\x13\n\x0fLOG_LEVEL_DEBUG\x10\x00\x12\x12\n\x0eLOG_LEVEL_INFO\x10\x01\x12\x15\n\x11LOG_LEVEL_WARNING\x10\x03\x12\x13\n\x0fLOG_LEVEL_ERROR\x10\x04\x12\x16\n\x12LOG_LEVEL_CRITICAL\x10\x05*\xb3\x01\n\x10ProfileEventType\x12\x1f\n\x1bPROFILE_EVENT_TYPE_REGISTER\x10\x00\x12\x1e\n\x1aPROFILE_EVENT_TYPE_CONNECT\x10\x04\x12!\n\x1dPROFILE_EVENT_TYPE_DEREGISTER\x10\x01\x12\x1b\n\x17PROFILE_EVENT_TYPE_SEND\x10\x02\x12\x1e\n\x1aPROFILE_EVENT_TYPE_RECEIVE\x10\x03*\xd3\x01\n\x10SettingValueType\x12\x1d\n\x19SETTING_VALUE_TYPE_STRING\x10\x00\x12\x1a\n\x16SETTING_VALUE_TYPE_INT\x10\x01\x12\x1c\n\x18SETTING_VALUE_TYPE_FLOAT\x10\x02\x12\x1b\n\x17SETTING_VALUE_TYPE_BOOL\x10\x03\x12!\n\x1dSETTING_VALUE_TYPE_LIST_FLOAT\x10\x04\x12&\n\"SETTING_VALUE_TYPE_LIST_LIST_FLOAT\x10\x05\x32\xf2\x04\n\rMuscleManager\x12[\n\x10SubmitLogMessage\x12#.muscle_manager_protocol.LogMessage\x1a\".muscle_manager_protocol.LogResult\x12_\n\x13SubmitProfileEvents\x12 .muscle_manager_protocol.Profile\x1a&.muscle_manager_protocol.ProfileResult\x12\x64\n\x0fRequestSettings\x12(.muscle_manager_protocol.SettingsRequest\x1a\'.muscle_manager_protocol.SettingsResult\x12m\n\x10RegisterInstance\x12,.muscle_manager_protocol.RegistrationRequest\x1a+.muscle_manager_protocol.RegistrationResult\x12Y\n\x0cRequestPeers\x12$.muscle_manager_protocol.PeerRequest\x1a#.muscle_manager_protocol.PeerResult\x12s\n\x12\x44\x65registerInstance\x12..muscle_manager_protocol.DeregistrationRequest\x1a-.muscle_manager_protocol.DeregistrationResultb\x06proto3')
  ,
  dependencies=[google_dot_protobuf_dot_timestamp__pb2.DESCRIPTOR,])

_RESULTSTATUS = _descriptor.EnumDescriptor(
  name='ResultStatus',
  full_name='muscle_manager_protocol.ResultStatus',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='RESULT_STATUS_SUCCESS', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='RESULT_STATUS_ERROR', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='RESULT_STATUS_PENDING', index=2, number=2,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=2088,
  serialized_end=2181,
)
_sym_db.RegisterEnumDescriptor(_RESULTSTATUS)

ResultStatus = enum_type_wrapper.EnumTypeWrapper(_RESULTSTATUS)
_OPERATOR = _descriptor.EnumDescriptor(
  name='Operator',
  full_name='muscle_manager_protocol.Operator',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='OPERATOR_NONE', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='OPERATOR_F_INIT', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='OPERATOR_O_I', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='OPERATOR_S', index=3, number=3,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='OPERATOR_B', index=4, number=4,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='OPERATOR_O_F', index=5, number=5,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='OPERATOR_MAP', index=6, number=6,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=2184,
  serialized_end=2320,
)
_sym_db.RegisterEnumDescriptor(_OPERATOR)

Operator = enum_type_wrapper.EnumTypeWrapper(_OPERATOR)
_LOGLEVEL = _descriptor.EnumDescriptor(
  name='LogLevel',
  full_name='muscle_manager_protocol.LogLevel',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='LOG_LEVEL_DEBUG', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='LOG_LEVEL_INFO', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='LOG_LEVEL_WARNING', index=2, number=3,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='LOG_LEVEL_ERROR', index=3, number=4,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='LOG_LEVEL_CRITICAL', index=4, number=5,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=2322,
  serialized_end=2441,
)
_sym_db.RegisterEnumDescriptor(_LOGLEVEL)

LogLevel = enum_type_wrapper.EnumTypeWrapper(_LOGLEVEL)
_PROFILEEVENTTYPE = _descriptor.EnumDescriptor(
  name='ProfileEventType',
  full_name='muscle_manager_protocol.ProfileEventType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='PROFILE_EVENT_TYPE_REGISTER', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PROFILE_EVENT_TYPE_CONNECT', index=1, number=4,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PROFILE_EVENT_TYPE_DEREGISTER', index=2, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PROFILE_EVENT_TYPE_SEND', index=3, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='PROFILE_EVENT_TYPE_RECEIVE', index=4, number=3,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=2444,
  serialized_end=2623,
)
_sym_db.RegisterEnumDescriptor(_PROFILEEVENTTYPE)

ProfileEventType = enum_type_wrapper.EnumTypeWrapper(_PROFILEEVENTTYPE)
_SETTINGVALUETYPE = _descriptor.EnumDescriptor(
  name='SettingValueType',
  full_name='muscle_manager_protocol.SettingValueType',
  filename=None,
  file=DESCRIPTOR,
  values=[
    _descriptor.EnumValueDescriptor(
      name='SETTING_VALUE_TYPE_STRING', index=0, number=0,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SETTING_VALUE_TYPE_INT', index=1, number=1,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SETTING_VALUE_TYPE_FLOAT', index=2, number=2,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SETTING_VALUE_TYPE_BOOL', index=3, number=3,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SETTING_VALUE_TYPE_LIST_FLOAT', index=4, number=4,
      serialized_options=None,
      type=None),
    _descriptor.EnumValueDescriptor(
      name='SETTING_VALUE_TYPE_LIST_LIST_FLOAT', index=5, number=5,
      serialized_options=None,
      type=None),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=2626,
  serialized_end=2837,
)
_sym_db.RegisterEnumDescriptor(_SETTINGVALUETYPE)

SettingValueType = enum_type_wrapper.EnumTypeWrapper(_SETTINGVALUETYPE)
RESULT_STATUS_SUCCESS = 0
RESULT_STATUS_ERROR = 1
RESULT_STATUS_PENDING = 2
OPERATOR_NONE = 0
OPERATOR_F_INIT = 1
OPERATOR_O_I = 2
OPERATOR_S = 3
OPERATOR_B = 4
OPERATOR_O_F = 5
OPERATOR_MAP = 6
LOG_LEVEL_DEBUG = 0
LOG_LEVEL_INFO = 1
LOG_LEVEL_WARNING = 3
LOG_LEVEL_ERROR = 4
LOG_LEVEL_CRITICAL = 5
PROFILE_EVENT_TYPE_REGISTER = 0
PROFILE_EVENT_TYPE_CONNECT = 4
PROFILE_EVENT_TYPE_DEREGISTER = 1
PROFILE_EVENT_TYPE_SEND = 2
PROFILE_EVENT_TYPE_RECEIVE = 3
SETTING_VALUE_TYPE_STRING = 0
SETTING_VALUE_TYPE_INT = 1
SETTING_VALUE_TYPE_FLOAT = 2
SETTING_VALUE_TYPE_BOOL = 3
SETTING_VALUE_TYPE_LIST_FLOAT = 4
SETTING_VALUE_TYPE_LIST_LIST_FLOAT = 5



_LOGMESSAGE = _descriptor.Descriptor(
  name='LogMessage',
  full_name='muscle_manager_protocol.LogMessage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='instance_id', full_name='muscle_manager_protocol.LogMessage.instance_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='timestamp', full_name='muscle_manager_protocol.LogMessage.timestamp', index=1,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='level', full_name='muscle_manager_protocol.LogMessage.level', index=2,
      number=4, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='text', full_name='muscle_manager_protocol.LogMessage.text', index=3,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=92,
  serialized_end=236,
)


_LOGRESULT = _descriptor.Descriptor(
  name='LogResult',
  full_name='muscle_manager_protocol.LogResult',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=238,
  serialized_end=249,
)


_PORT = _descriptor.Descriptor(
  name='Port',
  full_name='muscle_manager_protocol.Port',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='muscle_manager_protocol.Port.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='operator', full_name='muscle_manager_protocol.Port.operator', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=251,
  serialized_end=324,
)


_PROFILEEVENT = _descriptor.Descriptor(
  name='ProfileEvent',
  full_name='muscle_manager_protocol.ProfileEvent',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='instance_id', full_name='muscle_manager_protocol.ProfileEvent.instance_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='start_time', full_name='muscle_manager_protocol.ProfileEvent.start_time', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='stop_time', full_name='muscle_manager_protocol.ProfileEvent.stop_time', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='event_type', full_name='muscle_manager_protocol.ProfileEvent.event_type', index=3,
      number=4, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='port', full_name='muscle_manager_protocol.ProfileEvent.port', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='port_length', full_name='muscle_manager_protocol.ProfileEvent.port_length', index=5,
      number=6, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='slot', full_name='muscle_manager_protocol.ProfileEvent.slot', index=6,
      number=7, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='message_size', full_name='muscle_manager_protocol.ProfileEvent.message_size', index=7,
      number=8, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=327,
  serialized_end=622,
)


_PROFILE = _descriptor.Descriptor(
  name='Profile',
  full_name='muscle_manager_protocol.Profile',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='events', full_name='muscle_manager_protocol.Profile.events', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=624,
  serialized_end=688,
)


_PROFILERESULT = _descriptor.Descriptor(
  name='ProfileResult',
  full_name='muscle_manager_protocol.ProfileResult',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=690,
  serialized_end=705,
)


_REGISTRATIONREQUEST = _descriptor.Descriptor(
  name='RegistrationRequest',
  full_name='muscle_manager_protocol.RegistrationRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='instance_name', full_name='muscle_manager_protocol.RegistrationRequest.instance_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='network_locations', full_name='muscle_manager_protocol.RegistrationRequest.network_locations', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='ports', full_name='muscle_manager_protocol.RegistrationRequest.ports', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=707,
  serialized_end=824,
)


_REGISTRATIONRESULT = _descriptor.Descriptor(
  name='RegistrationResult',
  full_name='muscle_manager_protocol.RegistrationResult',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='muscle_manager_protocol.RegistrationResult.status', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='error_message', full_name='muscle_manager_protocol.RegistrationResult.error_message', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=826,
  serialized_end=924,
)


_CONDUIT = _descriptor.Descriptor(
  name='Conduit',
  full_name='muscle_manager_protocol.Conduit',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='sender', full_name='muscle_manager_protocol.Conduit.sender', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='receiver', full_name='muscle_manager_protocol.Conduit.receiver', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=926,
  serialized_end=969,
)


_PEERREQUEST = _descriptor.Descriptor(
  name='PeerRequest',
  full_name='muscle_manager_protocol.PeerRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='instance_name', full_name='muscle_manager_protocol.PeerRequest.instance_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=971,
  serialized_end=1007,
)


_PEERRESULT_PEERDIMENSIONS = _descriptor.Descriptor(
  name='PeerDimensions',
  full_name='muscle_manager_protocol.PeerResult.PeerDimensions',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='peer_name', full_name='muscle_manager_protocol.PeerResult.PeerDimensions.peer_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='dimensions', full_name='muscle_manager_protocol.PeerResult.PeerDimensions.dimensions', index=1,
      number=2, type=5, cpp_type=1, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1306,
  serialized_end=1361,
)

_PEERRESULT_PEERLOCATIONS = _descriptor.Descriptor(
  name='PeerLocations',
  full_name='muscle_manager_protocol.PeerResult.PeerLocations',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='instance_name', full_name='muscle_manager_protocol.PeerResult.PeerLocations.instance_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='locations', full_name='muscle_manager_protocol.PeerResult.PeerLocations.locations', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1363,
  serialized_end=1420,
)

_PEERRESULT = _descriptor.Descriptor(
  name='PeerResult',
  full_name='muscle_manager_protocol.PeerResult',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='muscle_manager_protocol.PeerResult.status', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='error_message', full_name='muscle_manager_protocol.PeerResult.error_message', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='conduits', full_name='muscle_manager_protocol.PeerResult.conduits', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='peer_dimensions', full_name='muscle_manager_protocol.PeerResult.peer_dimensions', index=3,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='peer_locations', full_name='muscle_manager_protocol.PeerResult.peer_locations', index=4,
      number=5, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[_PEERRESULT_PEERDIMENSIONS, _PEERRESULT_PEERLOCATIONS, ],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1010,
  serialized_end=1420,
)


_LISTOFDOUBLE = _descriptor.Descriptor(
  name='ListOfDouble',
  full_name='muscle_manager_protocol.ListOfDouble',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='values', full_name='muscle_manager_protocol.ListOfDouble.values', index=0,
      number=1, type=1, cpp_type=5, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1422,
  serialized_end=1452,
)


_LISTOFLISTOFDOUBLE = _descriptor.Descriptor(
  name='ListOfListOfDouble',
  full_name='muscle_manager_protocol.ListOfListOfDouble',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='values', full_name='muscle_manager_protocol.ListOfListOfDouble.values', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1454,
  serialized_end=1529,
)


_SETTING = _descriptor.Descriptor(
  name='Setting',
  full_name='muscle_manager_protocol.Setting',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='muscle_manager_protocol.Setting.name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value_type', full_name='muscle_manager_protocol.Setting.value_type', index=1,
      number=2, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value_string', full_name='muscle_manager_protocol.Setting.value_string', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value_int', full_name='muscle_manager_protocol.Setting.value_int', index=3,
      number=4, type=3, cpp_type=2, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value_float', full_name='muscle_manager_protocol.Setting.value_float', index=4,
      number=5, type=1, cpp_type=5, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value_bool', full_name='muscle_manager_protocol.Setting.value_bool', index=5,
      number=6, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value_list_float', full_name='muscle_manager_protocol.Setting.value_list_float', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='value_list_list_float', full_name='muscle_manager_protocol.Setting.value_list_list_float', index=7,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1532,
  serialized_end=1841,
)


_SETTINGSREQUEST = _descriptor.Descriptor(
  name='SettingsRequest',
  full_name='muscle_manager_protocol.SettingsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1843,
  serialized_end=1860,
)


_SETTINGSRESULT = _descriptor.Descriptor(
  name='SettingsResult',
  full_name='muscle_manager_protocol.SettingsResult',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='setting_values', full_name='muscle_manager_protocol.SettingsResult.setting_values', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1862,
  serialized_end=1936,
)


_DEREGISTRATIONREQUEST = _descriptor.Descriptor(
  name='DeregistrationRequest',
  full_name='muscle_manager_protocol.DeregistrationRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='instance_name', full_name='muscle_manager_protocol.DeregistrationRequest.instance_name', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1938,
  serialized_end=1984,
)


_DEREGISTRATIONRESULT = _descriptor.Descriptor(
  name='DeregistrationResult',
  full_name='muscle_manager_protocol.DeregistrationResult',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  fields=[
    _descriptor.FieldDescriptor(
      name='status', full_name='muscle_manager_protocol.DeregistrationResult.status', index=0,
      number=1, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
    _descriptor.FieldDescriptor(
      name='error_message', full_name='muscle_manager_protocol.DeregistrationResult.error_message', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=_b("").decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1986,
  serialized_end=2086,
)

_LOGMESSAGE.fields_by_name['timestamp'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_LOGMESSAGE.fields_by_name['level'].enum_type = _LOGLEVEL
_PORT.fields_by_name['operator'].enum_type = _OPERATOR
_PROFILEEVENT.fields_by_name['start_time'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_PROFILEEVENT.fields_by_name['stop_time'].message_type = google_dot_protobuf_dot_timestamp__pb2._TIMESTAMP
_PROFILEEVENT.fields_by_name['event_type'].enum_type = _PROFILEEVENTTYPE
_PROFILEEVENT.fields_by_name['port'].message_type = _PORT
_PROFILE.fields_by_name['events'].message_type = _PROFILEEVENT
_REGISTRATIONREQUEST.fields_by_name['ports'].message_type = _PORT
_REGISTRATIONRESULT.fields_by_name['status'].enum_type = _RESULTSTATUS
_PEERRESULT_PEERDIMENSIONS.containing_type = _PEERRESULT
_PEERRESULT_PEERLOCATIONS.containing_type = _PEERRESULT
_PEERRESULT.fields_by_name['status'].enum_type = _RESULTSTATUS
_PEERRESULT.fields_by_name['conduits'].message_type = _CONDUIT
_PEERRESULT.fields_by_name['peer_dimensions'].message_type = _PEERRESULT_PEERDIMENSIONS
_PEERRESULT.fields_by_name['peer_locations'].message_type = _PEERRESULT_PEERLOCATIONS
_LISTOFLISTOFDOUBLE.fields_by_name['values'].message_type = _LISTOFDOUBLE
_SETTING.fields_by_name['value_type'].enum_type = _SETTINGVALUETYPE
_SETTING.fields_by_name['value_list_float'].message_type = _LISTOFDOUBLE
_SETTING.fields_by_name['value_list_list_float'].message_type = _LISTOFLISTOFDOUBLE
_SETTINGSRESULT.fields_by_name['setting_values'].message_type = _SETTING
_DEREGISTRATIONRESULT.fields_by_name['status'].enum_type = _RESULTSTATUS
DESCRIPTOR.message_types_by_name['LogMessage'] = _LOGMESSAGE
DESCRIPTOR.message_types_by_name['LogResult'] = _LOGRESULT
DESCRIPTOR.message_types_by_name['Port'] = _PORT
DESCRIPTOR.message_types_by_name['ProfileEvent'] = _PROFILEEVENT
DESCRIPTOR.message_types_by_name['Profile'] = _PROFILE
DESCRIPTOR.message_types_by_name['ProfileResult'] = _PROFILERESULT
DESCRIPTOR.message_types_by_name['RegistrationRequest'] = _REGISTRATIONREQUEST
DESCRIPTOR.message_types_by_name['RegistrationResult'] = _REGISTRATIONRESULT
DESCRIPTOR.message_types_by_name['Conduit'] = _CONDUIT
DESCRIPTOR.message_types_by_name['PeerRequest'] = _PEERREQUEST
DESCRIPTOR.message_types_by_name['PeerResult'] = _PEERRESULT
DESCRIPTOR.message_types_by_name['ListOfDouble'] = _LISTOFDOUBLE
DESCRIPTOR.message_types_by_name['ListOfListOfDouble'] = _LISTOFLISTOFDOUBLE
DESCRIPTOR.message_types_by_name['Setting'] = _SETTING
DESCRIPTOR.message_types_by_name['SettingsRequest'] = _SETTINGSREQUEST
DESCRIPTOR.message_types_by_name['SettingsResult'] = _SETTINGSRESULT
DESCRIPTOR.message_types_by_name['DeregistrationRequest'] = _DEREGISTRATIONREQUEST
DESCRIPTOR.message_types_by_name['DeregistrationResult'] = _DEREGISTRATIONRESULT
DESCRIPTOR.enum_types_by_name['ResultStatus'] = _RESULTSTATUS
DESCRIPTOR.enum_types_by_name['Operator'] = _OPERATOR
DESCRIPTOR.enum_types_by_name['LogLevel'] = _LOGLEVEL
DESCRIPTOR.enum_types_by_name['ProfileEventType'] = _PROFILEEVENTTYPE
DESCRIPTOR.enum_types_by_name['SettingValueType'] = _SETTINGVALUETYPE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

LogMessage = _reflection.GeneratedProtocolMessageType('LogMessage', (_message.Message,), dict(
  DESCRIPTOR = _LOGMESSAGE,
  __module__ = 'muscle_manager_protocol_pb2'
  # @@protoc_insertion_point(class_scope:muscle_manager_protocol.LogMessage)
  ))
_sym_db.RegisterMessage(LogMessage)

LogResult = _reflection.GeneratedProtocolMessageType('LogResult', (_message.Message,), dict(
  DESCRIPTOR = _LOGRESULT,
  __module__ = 'muscle_manager_protocol_pb2'
  # @@protoc_insertion_point(class_scope:muscle_manager_protocol.LogResult)
  ))
_sym_db.RegisterMessage(LogResult)

Port = _reflection.GeneratedProtocolMessageType('Port', (_message.Message,), dict(
  DESCRIPTOR = _PORT,
  __module__ = 'muscle_manager_protocol_pb2'
  # @@protoc_insertion_point(class_scope:muscle_manager_protocol.Port)
  ))
_sym_db.RegisterMessage(Port)

ProfileEvent = _reflection.GeneratedProtocolMessageType('ProfileEvent', (_message.Message,), dict(
  DESCRIPTOR = _PROFILEEVENT,
  __module__ = 'muscle_manager_protocol_pb2'
  # @@protoc_insertion_point(class_scope:muscle_manager_protocol.ProfileEvent)
  ))
_sym_db.RegisterMessage(ProfileEvent)

Profile = _reflection.GeneratedProtocolMessageType('Profile', (_message.Message,), dict(
  DESCRIPTOR = _PROFILE,
  __module__ = 'muscle_manager_protocol_pb2'
  # @@protoc_insertion_point(class_scope:muscle_manager_protocol.Profile)
  ))
_sym_db.RegisterMessage(Profile)

ProfileResult = _reflection.GeneratedProtocolMessageType('ProfileResult', (_message.Message,), dict(
  DESCRIPTOR = _PROFILERESULT,
  __module__ = 'muscle_manager_protocol_pb2'
  # @@protoc_insertion_point(class_scope:muscle_manager_protocol.ProfileResult)
  ))
_sym_db.RegisterMessage(ProfileResult)

RegistrationRequest = _reflection.GeneratedProtocolMessageType('RegistrationRequest', (_message.Message,), dict(
  DESCRIPTOR = _REGISTRATIONREQUEST,
  __module__ = 'muscle_manager_protocol_pb2'
  # @@protoc_insertion_point(class_scope:muscle_manager_protocol.RegistrationRequest)
  ))
_sym_db.RegisterMessage(RegistrationRequest)

RegistrationResult = _reflection.GeneratedProtocolMessageType('RegistrationResult', (_message.Message,), dict(
  DESCRIPTOR = _REGISTRATIONRESULT,
  __module__ = 'muscle_manager_protocol_pb2'
  # @@protoc_insertion_point(class_scope:muscle_manager_protocol.RegistrationResult)
  ))
_sym_db.RegisterMessage(RegistrationResult)

Conduit = _reflection.GeneratedProtocolMessageType('Conduit', (_message.Message,), dict(
  DESCRIPTOR = _CONDUIT,
  __module__ = 'muscle_manager_protocol_pb2'
  # @@protoc_insertion_point(class_scope:muscle_manager_protocol.Conduit)
  ))
_sym_db.RegisterMessage(Conduit)

PeerRequest = _reflection.GeneratedProtocolMessageType('PeerRequest', (_message.Message,), dict(
  DESCRIPTOR = _PEERREQUEST,
  __module__ = 'muscle_manager_protocol_pb2'
  # @@protoc_insertion_point(class_scope:muscle_manager_protocol.PeerRequest)
  ))
_sym_db.RegisterMessage(PeerRequest)

PeerResult = _reflection.GeneratedProtocolMessageType('PeerResult', (_message.Message,), dict(

  PeerDimensions = _reflection.GeneratedProtocolMessageType('PeerDimensions', (_message.Message,), dict(
    DESCRIPTOR = _PEERRESULT_PEERDIMENSIONS,
    __module__ = 'muscle_manager_protocol_pb2'
    # @@protoc_insertion_point(class_scope:muscle_manager_protocol.PeerResult.PeerDimensions)
    ))
  ,

  PeerLocations = _reflection.GeneratedProtocolMessageType('PeerLocations', (_message.Message,), dict(
    DESCRIPTOR = _PEERRESULT_PEERLOCATIONS,
    __module__ = 'muscle_manager_protocol_pb2'
    # @@protoc_insertion_point(class_scope:muscle_manager_protocol.PeerResult.PeerLocations)
    ))
  ,
  DESCRIPTOR = _PEERRESULT,
  __module__ = 'muscle_manager_protocol_pb2'
  # @@protoc_insertion_point(class_scope:muscle_manager_protocol.PeerResult)
  ))
_sym_db.RegisterMessage(PeerResult)
_sym_db.RegisterMessage(PeerResult.PeerDimensions)
_sym_db.RegisterMessage(PeerResult.PeerLocations)

ListOfDouble = _reflection.GeneratedProtocolMessageType('ListOfDouble', (_message.Message,), dict(
  DESCRIPTOR = _LISTOFDOUBLE,
  __module__ = 'muscle_manager_protocol_pb2'
  # @@protoc_insertion_point(class_scope:muscle_manager_protocol.ListOfDouble)
  ))
_sym_db.RegisterMessage(ListOfDouble)

ListOfListOfDouble = _reflection.GeneratedProtocolMessageType('ListOfListOfDouble', (_message.Message,), dict(
  DESCRIPTOR = _LISTOFLISTOFDOUBLE,
  __module__ = 'muscle_manager_protocol_pb2'
  # @@protoc_insertion_point(class_scope:muscle_manager_protocol.ListOfListOfDouble)
  ))
_sym_db.RegisterMessage(ListOfListOfDouble)

Setting = _reflection.GeneratedProtocolMessageType('Setting', (_message.Message,), dict(
  DESCRIPTOR = _SETTING,
  __module__ = 'muscle_manager_protocol_pb2'
  # @@protoc_insertion_point(class_scope:muscle_manager_protocol.Setting)
  ))
_sym_db.RegisterMessage(Setting)

SettingsRequest = _reflection.GeneratedProtocolMessageType('SettingsRequest', (_message.Message,), dict(
  DESCRIPTOR = _SETTINGSREQUEST,
  __module__ = 'muscle_manager_protocol_pb2'
  # @@protoc_insertion_point(class_scope:muscle_manager_protocol.SettingsRequest)
  ))
_sym_db.RegisterMessage(SettingsRequest)

SettingsResult = _reflection.GeneratedProtocolMessageType('SettingsResult', (_message.Message,), dict(
  DESCRIPTOR = _SETTINGSRESULT,
  __module__ = 'muscle_manager_protocol_pb2'
  # @@protoc_insertion_point(class_scope:muscle_manager_protocol.SettingsResult)
  ))
_sym_db.RegisterMessage(SettingsResult)

DeregistrationRequest = _reflection.GeneratedProtocolMessageType('DeregistrationRequest', (_message.Message,), dict(
  DESCRIPTOR = _DEREGISTRATIONREQUEST,
  __module__ = 'muscle_manager_protocol_pb2'
  # @@protoc_insertion_point(class_scope:muscle_manager_protocol.DeregistrationRequest)
  ))
_sym_db.RegisterMessage(DeregistrationRequest)

DeregistrationResult = _reflection.GeneratedProtocolMessageType('DeregistrationResult', (_message.Message,), dict(
  DESCRIPTOR = _DEREGISTRATIONRESULT,
  __module__ = 'muscle_manager_protocol_pb2'
  # @@protoc_insertion_point(class_scope:muscle_manager_protocol.DeregistrationResult)
  ))
_sym_db.RegisterMessage(DeregistrationResult)



_MUSCLEMANAGER = _descriptor.ServiceDescriptor(
  name='MuscleManager',
  full_name='muscle_manager_protocol.MuscleManager',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  serialized_start=2840,
  serialized_end=3466,
  methods=[
  _descriptor.MethodDescriptor(
    name='SubmitLogMessage',
    full_name='muscle_manager_protocol.MuscleManager.SubmitLogMessage',
    index=0,
    containing_service=None,
    input_type=_LOGMESSAGE,
    output_type=_LOGRESULT,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='SubmitProfileEvents',
    full_name='muscle_manager_protocol.MuscleManager.SubmitProfileEvents',
    index=1,
    containing_service=None,
    input_type=_PROFILE,
    output_type=_PROFILERESULT,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='RequestSettings',
    full_name='muscle_manager_protocol.MuscleManager.RequestSettings',
    index=2,
    containing_service=None,
    input_type=_SETTINGSREQUEST,
    output_type=_SETTINGSRESULT,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='RegisterInstance',
    full_name='muscle_manager_protocol.MuscleManager.RegisterInstance',
    index=3,
    containing_service=None,
    input_type=_REGISTRATIONREQUEST,
    output_type=_REGISTRATIONRESULT,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='RequestPeers',
    full_name='muscle_manager_protocol.MuscleManager.RequestPeers',
    index=4,
    containing_service=None,
    input_type=_PEERREQUEST,
    output_type=_PEERRESULT,
    serialized_options=None,
  ),
  _descriptor.MethodDescriptor(
    name='DeregisterInstance',
    full_name='muscle_manager_protocol.MuscleManager.DeregisterInstance',
    index=5,
    containing_service=None,
    input_type=_DEREGISTRATIONREQUEST,
    output_type=_DEREGISTRATIONRESULT,
    serialized_options=None,
  ),
])
_sym_db.RegisterServiceDescriptor(_MUSCLEMANAGER)

DESCRIPTOR.services_by_name['MuscleManager'] = _MUSCLEMANAGER

# @@protoc_insertion_point(module_scope)
